--*************************************************
--**	Copyright (C) 2003 SENG digitale Syteme GmbH
--**
--**	SENG digitale Systeme GmbH
--**	Im Bruckwasen 35
--**	D 73037 Goeppingen
--**	Germany
--**	http://www.seng.de
--*************************************************

-- CPU and flash memory address decoding unit
-- use flash memory for program memory and MCS-52 basic tokens 


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL; 


entity CpuDec is
	Port (
		A: in STD_LOGIC_VECTOR(15 downto 0);
		nPSEN,nRD,nWR,RESET: in STD_LOGIC;
		nRCE,nROE,nOeDat,WeRam,CEi2cD,CEi2cC: out STD_LOGIC;
		RdSel: out STD_LOGIC_VECTOR(1 downto 0) 
	);
end CpuDec;


architecture decode_logic of CpuDec is

begin

	Sel_flash: process(A,nPSEN,nRD,RESET)
	begin
		if (RESET ='1') then
			nRCE <= '1'; -- 8052 reset
			nROE <= '1';
		else
			if ( nPSEN = '0') then
			   nRCE <= '0'; -- programm storage enabled, max. 64Kx8
				nROE <= '0';
			else
				if (nRD = '0') and (a >= x"8000") and (a < x"9000") then
					nRCE <= '0'; -- read basic programm storage from flash, size 4Kx8
					nROE <= '0';
				else
					nRCE <= '1'; -- I/O range
					nROE <= '1';
				end if;
			end if;
		end if;
	end process Sel_flash;

	
	Oe_Data: process(A,nRD,RESET)
	begin
		if ( nRD = '0') and (A >= x"0000") and (A < x"1000") and (RESET ='0')then
			nOeDat <= '0'; -- read basic programm storage from LCA-SRAM, size 4Kx8
		else
			if ( nRD = '0') and (A = x"fb10") and (RESET ='0')then
				nOeDat <= '0'; -- read i2cSDA, size 1x8
			else
				if ( nRD = '0') and (A = x"fb00") and (RESET ='0')then
					nOeDat <= '0'; -- read i2cSCL, size 1x8
				else
					nOeDat <= '1'; -- no read
				end if;
			end if;
		end if;
	end process Oe_Data;

 
	OmuxData: process(A)
	begin
		if (A >= x"0000") and (A < x"1000") then
			RdSel <= "00"; -- read basic programm storage from FPGA-SRAM, size 4Kx8
		else
			if (A = x"fb10") then
				RdSel <= "01"; -- read i2cSDA, size 1x8
			else
				RdSel <= "10"; -- read i2cSCL, size 1x8
			end if;
		end if;
	end process OmuxData;

	
	WRenable: process(A,nWR)
	begin
		if ( nWr = '0') and (A >= x"0000") and (A < x"1000") then
			WeRam <= '1'; -- enable WE from RAM
			CEi2cD <= '0'; -- disable CE from i2c SDA
			CEi2cC <= '0'; -- disable CE from i2c SCL
		else
			if ( nWr = '0') and (A = x"fb10") then
				WeRam <= '0'; -- disable WE from RAM
				CEi2cD <= '1'; -- enable CE from i2c SDA
				CEi2cC <= '0'; -- disable CE from i2c SCL
			else
				if ( nWr = '0') and (A = x"fb00") then
					WeRam <= '0'; --  disable WE from RAM
					CEi2cD <= '0'; -- disable CE from i2c SDA
					CEi2cC <= '1'; -- enable CE from i2c SCL
				else						
					WeRam <= '0'; -- disable WE from RAM
					CEi2cD <= '0'; -- disable CE from i2c SDA
					CEi2cC <= '0'; -- disable CE from i2c SCL	
				end if;
			end if;
		end if;
	end process WRenable;


end decode_logic;
