--****************************************************
--**	Copyright (C) 2002 SENG digitale Syteme GmbH
--**
--**	SENG digitale Systeme GmbH
--**	Im Bruckwasen 35
--**	D 73037 Goeppingen
--**	Germany
--**	http://www.seng.de
--****************************************************

-- EPP(enhanced parallel port) control, state-machine
-- state machine generated wait times have to be modified according to clockrate
-- designed for 11MHz clockrate, generating 180ns nWR pulse
-- design based on IEE Std 1284-1994
-- module: EPP_Control
-- Company: SENG digitale Systeme GmbH
-- Designer: Peter Seng
-- Implementation: for FPGA
-- Date: 31.oct.2002
-- Last change: 26.nov.2002


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity EPP_Control is
    Port ( 
	 	CLK: in STD_LOGIC;		
		nDstr, nDstrF, nAstr,AstrF, nWriteF: in STD_LOGIC;
		nRd, nWr, Aen, nWait, EPPrst: out STD_LOGIC
	 );
end EPP_Control;

architecture Behavioral of EPP_Control is
	signal state: STD_LOGIC_VECTOR(2 downto 0);
begin
	
	ResetEPP: process(CLK)
	begin
		if (CLK='1' and CLK'event) then --CLK rising edge
			if (nDstrF = '0' and AstrF = '1' and nWriteF = '1') then
	 	   	EPPrst <= '1'; -- reset active
			else
				EPPrst <= '0'; -- no reset
			end if;
		end if;
	end process;

	StateCount: process(CLK,nDstr,nAstr)
	begin
		if (nDstr = '1' and nAstr = '1') then --asynchronous setings
 	   	state <= "000";
			nRD <= '1';
			nWR <= '1';
			Aen <= '0';
			nWait <= '0';
		elsif (CLK='1' and CLK'event) then --CLK rising edge
			if (state = "000") THEN -- state 0
				nWait <= '0';
				if (nDstrF = '0' and nWriteF = '0') then -- no data write phase, data stabilizing
					nRD <= '1';
					nWR <= '1';
					Aen <= '0';
					state <= state + "001";
				elsif (nDstrF = '0' and nWriteF = '1') then -- data read phase
					nRD <= '0';
					nWR <= '1';
					Aen <= '0';
					state <= state + "001";
				elsif (AstrF = '1' and nWriteF = '0') then -- no address write phase, data stabilizing
					nRD <= '1';
					nWR <= '1';
					Aen <= '0';
					state <= state + "001";
				else -- not active
					nRD <= '1';
					nWR <= '1';
					Aen <= '0';
				end if;
			elsif (state > "000" and state < "011") THEN -- state 1..2, data valid
				nWait <= '0';
				if (nDstrF = '0' and nWriteF = '0') then -- data write phase
					nRD <= '1';
					nWR <= '0';
					Aen <= '0';
					state <= state + "001";
				elsif (nDstrF = '0' and nWriteF = '1') then -- data read phase
					nRD <= '0';
					nWR <= '1';
					Aen <= '0';
					state <= state + "001";
				elsif (AstrF = '1' and nWriteF = '0') then -- address write phase
					nRD <= '1';
					nWR <= '1';
					Aen <= '1';
					state <= state + "001";
				else -- not active
					nRD <= '1';
					nWR <= '1';
					Aen <= '0';
				end if;
			elsif (state = "011") THEN -- state 3, endless loop till asychronous end
				state <= "011"; -- state is constant --> endless loop
				nWait <= '1'; -- terminate wait state
				if (nDstrF = '0' and nWriteF = '1') then -- continue data read phase
					nRD <= '0';
					nWR <= '1';
					Aen <= '0';
				else -- write phase
					nRD <= '1';
					nWR <= '1';
					Aen <= '0';
				end if;
			else -- for safety
		      state <= "000";
				nRD <= '1';
				nWR <= '1';
				Aen <= '0';
				nWait <= '0';
			end if;
	   end if;
	end process;			

end Behavioral;






